import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A special node that can be dragged and can move by itself.
 */
public class StartNode extends Node
{
    private int step;
    private int originY;
    
    private Slider ampSlider;
    private Slider freqSlider;
    private Switch oscSwitch;
    private Switch lineLoopSwitch;
    private boolean lineLoopState;

    public StartNode(Slider amp, Slider freq, Switch osc, Switch onSwitch, Switch lls)
    {
        super(null, onSwitch);
        ampSlider = amp;
        freqSlider = freq;
        oscSwitch = osc;
        lineLoopSwitch = lls;
        lineLoopState = lineLoopSwitch.isOn();
    }
    
    public void addedToWorld(World w)
    {
        originY = getY();
    }
    
    public void act() 
    {
        if (onSwitch.isOn()) {
            // first check to see if we need to rearrange the balls
            if (lineLoopSwitch.isOn() != lineLoopState) {
                // state change for the balls arrangement
                lineLoopState = lineLoopSwitch.isOn();
                WaveLab theWorld = (WaveLab) this.getWorld();
                theWorld.arrangeBalls();
            }
                
            if (oscSwitch.isOn()) {
                if ( !ampSlider.isEnabled() ) {
                    ampSlider.setEnabled(true);
                    freqSlider.setEnabled(true);
                }
                doOscillate();
            }
            else {
                if ( ampSlider.isEnabled() ) {
                    ampSlider.setEnabled(false);
                    freqSlider.setEnabled(false);
                }
                doDrag();
            }
        }
    }    

    private void doOscillate()
    {
        step = (step + freqSlider.getValue()) % 360;
        int y = (int) (originY + Math.sin(Math.toRadians(step)) * ampSlider.getValue());
        setLocation (getX(), y);
    }    
    
    private void doDrag()
    {
        if (Greenfoot.mouseDragged(this)) {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            setLocation(getX(), mouse.getY());
        }
    }

}
