import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.ArrayList;
import java.awt.Color;

/**
 * Write a description of class MyWorld here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class WaveLab extends World
{
    static private final int NO_BALLS = 59;
    static private final int SPACING = 14;
    static private final int LINE_START_X = 80;
    static private final int LINE_START_Y = 300;
    
    private Node[] balls;
    
    private Slider amplitudeSlider;
    private Slider frequencySlider;
    private Slider dampingSlider;
    private Switch oscillateSwitch;
    private Switch onSwitch;
    private Switch lineLoopSwitch;
    
    /**
     * Constructor for objects of class MyWorld.
     * 
     */
    public WaveLab()
    {    
        super(960, 500, 1);

        GreenfootImage bg = getBackground();
        bg.setColor( new Color (0, 0, 0, 128));
        bg.drawLine(LINE_START_X, LINE_START_Y, LINE_START_X+820, LINE_START_Y);

        createControls();
        balls = createBalls();
    }

    public void act() 
    {
        for (Node node : balls) {
            node.applyForce(dampingSlider.getValue() / 100.0);
        }
    }
   
    private void createControls() 
    {
        oscillateSwitch = new Switch ("Manual", "Oscillate");
        addObject (oscillateSwitch, 100, 65);
        
        amplitudeSlider = new Slider ("Amplitude", 0, 100);
        addObject (amplitudeSlider, 260, 65);

        frequencySlider = new Slider ("Frequency", 3, 25);
        addObject (frequencySlider, 470, 65);
        
        dampingSlider = new Slider ("Damping", 0, 10);
        addObject (dampingSlider, 680, 65);
        
        onSwitch = new Switch ("Pause", "Go");
        onSwitch.toggle();
        addObject (onSwitch, 840, 65);
        
        lineLoopSwitch = new Switch ("Line", "Loop");
        addObject(lineLoopSwitch, 900, 65);
        
        addObject (new Sign(), 100, 390);

    }
    
    private Node[] createBalls() 
    {
        Node [] nodes = new Node[NO_BALLS];
        Node start = new StartNode(amplitudeSlider, frequencySlider, oscillateSwitch, onSwitch, lineLoopSwitch);
        addObject (start, LINE_START_X - SPACING - 1, LINE_START_Y);
        Node prev = start;
        for (int i = 0; i < NO_BALLS; i++) {
            nodes[i] = new Node(prev, onSwitch);
            addObject (nodes[i], LINE_START_X + i*SPACING, LINE_START_Y);
            prev = nodes[i];
        }
        return nodes;
    }
    
    /**
     * Arrange the balls according to the lineLoop switch.  Line means the original line.
     * Loop means that the leftmost red ball is moved to the rightmost position, with the
     * rest of the balls following like a string looped back on itself.
     */
    public void arrangeBalls()
    {
        if (lineLoopSwitch.isOn()) {
            // arrange the balls into a loop
            for (int i = 0; i < NO_BALLS/2; i++) {
                double opposite = balls[NO_BALLS-1-i].getExactX();
                balls[i].setLocation(opposite, balls[i].getExactY());
            }
        } else {
            // arrange the balls into a straight line
            for (int i = 0; i < NO_BALLS/2; i++) {
                balls[i].setLocation(LINE_START_X + i*SPACING, balls[i].getExactY());
            }
        }
    }
}
